Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 276570)
+++ channels/chan_sip.c	(working copy)
@@ -20410,6 +20410,36 @@
 	return different;
 }
 
+static int sip_uri_domain_cmp(const char *host1, const char *host2)
+{
+	struct ast_sockaddr addr1;
+	struct ast_sockaddr addr2;
+	int addr1_parsed;
+	int addr2_parsed;
+
+	addr1_parsed = ast_sockaddr_parse(&addr1, host1, 0);
+	addr2_parsed = ast_sockaddr_parse(&addr2, host2, 0);
+
+	if (addr1_parsed ^ addr2_parsed) {
+		/* One domain was an IP address and the other had
+		 * a host name. FAIL!
+		 */
+		return 1;
+	}
+
+	/* Both are host names. A string comparison will work
+	 * perfectly here.
+	 */
+	if (!addr1_parsed) {
+		if (strcasecmp(host1, host2)) {
+			return 1;
+		}
+	}
+
+	/* Both contain IP addresses */
+	return ast_sockaddr_cmp(&addr1, &addr2);
+}
+
 static int sip_uri_cmp(const char *input1, const char *input2)
 {
 	char *uri1 = ast_strdupa(input1);
@@ -20421,6 +20451,11 @@
 	char *headers1;
 	char *headers2;
 
+	/* XXX It would be really nice if we could just use parse_uri_full() here
+	 * to separate the components of the URI, but unfortunately it is written
+	 * in a way that can cause URI parameters to be discarded.
+	 */
+
 	/* Strip off "sip:" from the URI. We know this is present
 	 * because it was checked back in parse_request()
 	 */
@@ -20469,17 +20504,7 @@
 		*headers2++ = '\0';
 	}
 
-	/* Now the host/port are properly isolated. We can get by with a string comparison
-	 * because the SIP URI checking rules have some interesting exceptions that make
-	 * this possible. I will note 2 in particular
-	 * 1. hostnames which resolve to the same IP address as well as a hostname and its
-	 *    IP address are not considered a match with SIP URI's.
-	 * 2. If one URI specifies a port and the other does not, then the URIs do not match.
-	 *    This includes if one URI explicitly contains port 5060 and the other implies it
-	 *    by not having a port specified.
-	 */
-
-	if (strcasecmp(host1, host2)) {
+	if (sip_uri_domain_cmp(host1, host2)) {
 		return 1;
 	}
 
